//   Copyright 2010 Schnatterplatsch
//
//   This file is part of Fy.WoD!.
//
//   Fy.WoD!. is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Fy.WoD!. is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Fy.WoD!.  If not, see <http://www.gnu.org/licenses/>.
//


#ifndef _CONTROL_H
#define _CONTROL_H

  #ifdef __cplusplus
    extern "C" {
  #endif

  #include <stdio.h>
  #include <stdlib.h>
  #include <SDL/SDL.h>

  #include "sfx.h"

  #ifdef GP2X

    enum KEYLIST
    {
      KEY_UP,
      KEY_UP_LEFT,
      KEY_LEFT,
      KEY_DOWN_LEFT,
      KEY_DOWN,
      KEY_DOWN_RIGHT ,
      KEY_RIGHT,
      KEY_UP_RIGHT,
      KEY_MENU,
      KEY_SELECT,
      KEY_SHLEFT,
      KEY_SHRIGHT,
      KEY_A,
      KEY_B,
      KEY_X,
      KEY_Y,
      KEY_VUP,
      KEY_VDOWN
    };

  #elif defined(DINGOO)

    enum KEYLIST
    {
      KEY_UP  = SDLK_UP,
      KEY_UP_LEFT = SDLK_UP ,
      KEY_LEFT = SDLK_LEFT ,
      KEY_DOWN_LEFT = SDLK_LEFT ,
      KEY_DOWN = SDLK_DOWN ,
      KEY_DOWN_RIGHT = SDLK_DOWN ,
      KEY_RIGHT = SDLK_RIGHT ,
      KEY_UP_RIGHT = SDLK_RIGHT ,
      KEY_MENU = SDLK_RETURN ,
      KEY_SELECT = SDLK_ESCAPE ,
      KEY_SHLEFT = SDLK_TAB ,
      KEY_SHRIGHT = SDLK_BACKSPACE ,
// Keys A and B are switched because of different GP2X vs DINGOO button layout.
// Key A is in fact button B on the Dingoo and vice versa.
      KEY_A = SDLK_LALT ,
      KEY_B = SDLK_LCTRL ,
      KEY_X = SDLK_SPACE ,
      KEY_Y = SDLK_LSHIFT ,
// DINGOO lacks volume buttons, so this is left unmodified.
      KEY_VUP = SDLK_KP_PLUS,
      KEY_VDOWN = SDLK_KP_MINUS
    };


  #else

    enum KEYLIST
    {
      KEY_UP  = SDLK_UP,
      KEY_UP_LEFT = SDLK_UP ,
      KEY_LEFT = SDLK_LEFT ,
      KEY_DOWN_LEFT = SDLK_LEFT ,
      KEY_DOWN = SDLK_DOWN ,
      KEY_DOWN_RIGHT = SDLK_DOWN ,
      KEY_RIGHT = SDLK_RIGHT ,
      KEY_UP_RIGHT = SDLK_RIGHT ,
      KEY_MENU = SDLK_KP_ENTER ,
      KEY_SELECT = SDLK_KP_PERIOD ,
      KEY_SHLEFT = SDLK_q ,
      KEY_SHRIGHT = SDLK_e ,
      KEY_A = SDLK_a ,
      KEY_B = SDLK_d ,
      KEY_X = SDLK_s ,
      KEY_Y = SDLK_w ,
      KEY_VUP = SDLK_KP_PLUS,
      KEY_VDOWN = SDLK_KP_MINUS
    };

  #endif

  struct cntrlKeyEvent
  {

    int pressed;
    int newpressed, released;

  };

  struct cntrlMouseEvent
  {

    int x,y;
    int pressed;
    int newpressed, released;

  };

  struct
  {

    struct cntrlKeyEvent up, right, down, left ;
    struct cntrlKeyEvent y, b, x, a;
    struct cntrlKeyEvent shleft, shright;
    struct cntrlKeyEvent menu, select;
    struct cntrlKeyEvent vdown, vup;

    struct cntrlMouseEvent mouse;

  }cntrl_key;

  SDL_Joystick *cntrl_joystick;
  SDL_Event cntrl_event;


  void cntrlInit();
  void cntrlUpdate();

  #ifdef __cplusplus
    }
  #endif

#endif
